# -*- coding: utf-8 -*-
# Copyright (c) 2003-2006 Ingeniweb SAS
#
# This software is subject to the provisions of the GNU General Public
# License, Version 2.0 (GPL).  A copy of the GPL should accompany this
# distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
# AGAINST INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE
#
# More details in the ``LICENSE`` file included in this package.
"""
Monkey patch for ZWiki (from version 0.51)
$Id: ZWiki4PlonePatch.py 35072 2006-12-26 16:24:20Z glenfant $
"""

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

import logging
import types

try:
    from Products.PlacelessTranslationService.MessageID import MessageIDFactory
except ImportError, e:
    MessageIDFactory = None
    from Products.PlacelessTranslationService import translate

try:
    # Old ZWikis
    from Products.ZWiki import I18nSupport as ZWI18n
except ImportError, e:
    # From ZWiki 0.58.0 (?)
    from Products.ZWiki import I18n as ZWI18n

from Products.ZWiki.Utils import formattedTraceback

from Products.ZWikiFolder.config import MAILOUT_CHARSET, SITE_CHARSET

def ourcontext(s):
    """
    XXX wrapper ensuring proper utf-8 http header when _ is used.

    Called for every i18n string, performance sensitive ?
    When there is no request, do nothing so unit tests can run.
    """

    logger = logging.getLogger('ZWikiFolder')
    logger.info("ZWiki.I18nSupport monkey patch works")

    try:
        REQUEST = get_request()
        if not REQUEST:
            return str(s)
        else:
            contenttype = REQUEST.RESPONSE.getHeader('content-type') or 'text/html'
            if not 'charset' in contenttype:
                REQUEST.RESPONSE.setHeader('Content-Type','%s; charset=%s' % (contenttype, SITE_CHARSET))
            if MessageIDFactory:
                return str(MessageIDFactory('zwiki')(s))
            else:
                return str(translate('zwiki', s))
        
    except AttributeError:
        return str(s)

ZWI18n._ = ourcontext

try:
    from Products.ZWiki.Mail import MailSupport as ZWMailSupport
except ImportError, e:
    from Products.ZWiki.Mail import PageMailSupport as ZWMailSupport


def ourSendMailTo(self, recipients, text, REQUEST,
                  subjectSuffix='',
                  subject='',
                  message_id=None,
                  in_reply_to=None,
                  to=None,
                  exclude_address=None):
    """
    Send a mail-out containing text to a list of email addresses.

    If mail-out is not configured in this wiki or there are no valid
    recipients, do nothing.  Catch and log any errors when sending
    mail.
    
    XXX templatize ?
    XXX ezmlm won't deliver with precedence: bulk, which these are, what to do
    FIXME: use email package in the future (more pythonic)
    """
    logger = logging.getLogger('ZWikiFolder')
    
    if not self.isMailoutEnabled():
        return

    # gather bits and pieces
    msgid = message_id or self.messageIdFromTime(self.ZopeTime())
    tohdr = to or self.toHeader()

    # do some last-minute winnowing-out of recipients we don't want to
    # send to (and mail we don't want to send)

    # help mailin.py to exclude a list address to avoid a loop
    try:
        recipients.remove(exclude_address)
    except ValueError:
        pass

    if not recipients:
        return
    
    # some lists may deliver duplicated addresses twice; try to avoid
    # unnecessary
    #unique = []
    #for r in recipients:
    #    if not r in unique:
    #        unique.append(r)
    #if self.toProperty() in unique: unique.remove(self.toProperty())


    # More easy to debug
    msg_dict = {}
    msg_dict['from'] = encodeToMail(self.fromHeader(REQUEST))
    msg_dict['replyto'] = encodeToMail(self.replyToHeader())
    msg_dict['to'] = encodeToMail(tohdr)
    msg_dict['bcc'] = encodeToMail(self.bccHeader(recipients))
    msg_dict['subject'] = ourSubjectHeader(self, subject, subjectSuffix)
    msg_dict['msgid'] = encodeToMail(msgid)
    msg_dict['inreplyto'] = encodeToMail((in_reply_to and '\nIn-reply-to: %s' % in_reply_to) or '')
    msg_dict['zwiki_version'] = self.zwiki_version()
    msg_dict['beenthere'] = encodeToMail(self.xBeenThereHeader())
    msg_dict['listid'] = encodeToMail(self.listIdHeader())
    msg_dict['listpost'] = encodeToMail(self.listPostHeader())
    page_url = encodeToMail(self.pageUrl())
    msg_dict['listsubscribe'] = page_url
    msg_dict['listunsubscribe'] = page_url
    msg_dict['listarchive'] = page_url
    msg_dict['listhelp'] = encodeToMail(self.wikiUrl())
    msg_dict['text'] = encodeToMail(text)
    msg_dict['signature'] = encodeToMail(self.signature(msgid))
    msg_dict['charset'] = MAILOUT_CHARSET

    # Recoding all that's in Unicode

    msg = """\
From: %(from)s
Reply-To: %(replyto)s
To: %(to)s
Bcc: %(bcc)s
Subject: %(subject)s
Message-ID: %(msgid)s%(inreplyto)s
X-Zwiki-Version: %(zwiki_version)s
X-BeenThere: %(beenthere)s
List-Id: %(listid)s
List-Post: %(listpost)s
List-Subscribe: <%(listsubscribe)s/subscribeform>
List-Unsubscribe: <%(listunsubscribe)s/subscribeform>
List-Archive: <%(listarchive)s>
List-Help: <%(listhelp)s>
Content-Type: text/plain; charset="%(charset)s"

%(text)s

%(signature)s
""" % msg_dict

    try:
        logger.info('sent mail:\n%s', msg)
        self.mailhost().send(msg)
        logger.info('sent mail to subscribers:\nTo: %s\nBcc: %s' %
                    (tohdr,self.bccHeader(recipients)))
    # if there is any failure, notify admin or log
    except:
        logger.warning('failed to send mail to %s: %s' % (recipients,
                                                          formattedTraceback()))
        admin = getattr(self.folder(),'mail_admin',None)
        error_suffix = 'ERROR, subscriber mailout failed'
        # test subjectSuffix to avoid infinite recursion on sendMailTo
        if admin and subjectSuffix != error_suffix:
            try:
                self.sendMailTo(
                    [],text,REQUEST,
                    subjectSuffix=error_suffix,
                    to=admin)
            except:
                logger.warning('failed to send error report to admin: %s' % \
                               formattedTraceback())

ZWMailSupport.sendMailTo = ourSendMailTo

def ourSubjectHeader(page, subject='', subjectSuffix=''):
    """
    Replacement for Mail.PageMailSupport.subjectHeader()
    @return: encoded subject header (rfc822)
    """

    prefix = getattr(page.folder(), 'mail_subject_prefix', '').strip()

    if getattr(page.folder(),'mail_page_name',1):
        # we will add the page name
        if (page.issueNumber()
            and (page.getPhysicalRoot().ZopeTime()-page.creationTime()) > 30.0/24/60/60
            and not getattr(page.folder(),'mail_issue_name',0)):
            # we will truncate it to just the issue number
            pagename = '[#%s] ' % page.issueNumber()
        else:
            pagename = '[%s] ' % page.pageName()
    else:
        # page name has been suppressed
        pagename = ''
    header = (encodeToMail(prefix) + encodeToMail(pagename) +
              encodeToMail(subject) + encodeToMail(subjectSuffix.strip()))
    return header


def encodeToMail(text):
    """
    Safely encodes text to the email charset
    @param text: The text to encode (unicode or string)
    @return: the encoded text
    """

    if type(text) is types.UnicodeType:
        text = text.encode(MAILOUT_CHARSET, 'replace')
    else:
        # Supposed to be a string
        text = str(text)
        text = unicode(text, SITE_CHARSET, 'replace').encode(MAILOUT_CHARSET, 'replace')
    return text

